# -*- coding: utf-8 -*-
import time

from monitor import Monitor


class RemoteStatsClient(object):
    def __init__(self, server, hostname, interval):
        self.server = server
        self.hostname = hostname
        self.interval = interval

    def run(self):
        monitors = [m(self.server, self.hostname, self.interval)
                    for m in Monitor.monitors()]
        while True:
            start_time = time.time()

            map(lambda x: x.measure(), monitors)
            map(lambda x: x.flush(), monitors)

            time_wait = start_time + self.interval - time.time()
            time.sleep(time_wait if time_wait > 0 else 0)



# -*- coding: utf-8 -*-
import time
import psutil
import requests


class Monitor(object):
    def __init__(self, server, hostname, interval):
        self.server = server
        self.hostname = hostname
        self.interval = interval
        self.last_samples = []
        self.id = type(self).__name__ + '_' + hostname
        self._plot_registered = False

    @staticmethod
    def monitors():
        """List all monitors in this module."""
        def all_subclasses(cls):
            sub_classes = cls.__subclasses__()
            return sub_classes + [g for s in cls.__subclasses__()
                                  for g in all_subclasses(s)]
        current_module = __name__
        monitor_classes = all_subclasses(Monitor)
        monitor_classes = filter(lambda x: x.__module__ == current_module,
                                 monitor_classes)
        return monitor_classes

    def measure(self):
        """Measure a datapoint and stores it in the last_samples list."""
        raise NotImplementedError("measure must be implemented in a subclass")

    def flush(self):
        """Tries to send the last samples to the server."""
        if self.plot_registered:
            try:
                r = requests.put(self.server, json={
                    'id': self.id, 'measures': self.last_samples})
            except requests.exceptions.RequestException:
                return
            if r.status_code == 200:
                self.last_samples = []
            elif r.status_code == 404:
                self._plot_registered = False

    def _add_sample(self, value):
        current_time = int(time.time()*1000)
        sample = [current_time, value]
        self.last_samples.append(sample)

    @property
    def plot_registered(self):
        if not self._plot_registered:
            req_json = {'id': self.id,
                        'hostname': self.hostname,
                        'description': self.description,
                        'interval': self.interval*1000}
            try:
                requests.post(self.server, json=req_json)
            except requests.exceptions.RequestException:
                return False
            else:
                self._plot_registered = True
        return self._plot_registered

    @property
    def description(self):
        raise NotImplementedError("description must be implemented in a"
                                  "subclass")


class MemoryMonitor(Monitor):
    def measure(self):
        self._add_sample(psutil.virtual_memory().percent)

    @property
    def description(self):
        return 'Memory (%)'


class CpuMonitor(Monitor):
    def measure(self):
        self._add_sample(psutil.cpu_percent())

    @property
    def description(self):
        return 'CPU (%)'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# import flake8 as flake8
# import tox as tox

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'psutil',
    'requests'
]

test_requirements = [
    'flake8',
    'tox',
    'mock',
    'coverage'
]

setup(
    name='remotestatsclient',
    version='1.0.3',
    description="RemoteStats makes possible to inspect realtime information "
                "about a remote machine using a browser. This is the client"
                " to be installed in such machine.",
    long_description=readme + '\n\n' + history,
    author="Hugo Sadok",
    author_email='hugo@sadok.com.br',
    url='https://github.com/hugombarreto/remotestatsclient',
    packages=[
        'remotestatsclient',
    ],
    package_dir={'remotestatsclient':
                 'remotestatsclient'},
    entry_points={'console_scripts':
                      ['remotestatsclient=remotestatsclient.__main__:main']},
    include_package_data=True,
    install_requires=requirements,
    license="ISCL",
    zip_safe=False,
    keywords='remotestatsclient',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: ISC License (ISCL)',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='tests',
    tests_require=test_requirements
)



# -*- coding: utf-8 -*-
import unittest
import requests
from mock import MagicMock

from remotestatsclient.monitor import Monitor


class TestMonitor(unittest.TestCase):

    def test_measure(self):
        self.assertRaises(NotImplementedError,
                          Monitor('http://localhost', 'computerA', 5).measure)

    def test_add_sample(self):
        monitor = Monitor('http://localhost', 'computerA', 5)
        monitor._add_sample(10)
        self.assertEquals(monitor.last_samples[-1][1], 10)

    def test_subclasses_measure(self):
        monitors = [m('http://localhost', 'computerA', 5)
                    for m in Monitor.monitors()]
        for m in monitors:
            m.measure()
            self.assertEqual(len(m.last_samples), 1)
            self.assertIsInstance(m.last_samples[-1], list)
            self.assertEqual(len(m.last_samples[-1]), 2)


class SubMonitor(Monitor):
    def __init__(self, *args):
        super(SubMonitor, self).__init__('http://localhost', 'computerA', 5)

    def measure(self):
        return 1

    @property
    def description(self):
        return 'Foo (m/s)'


class TestMonitorSubclass(unittest.TestCase):
    @staticmethod
    def _mock_requests_with_code(code):
        requests.post = MagicMock()
        requests.put = MagicMock()
        return_mock = MagicMock()
        return_mock.status_code = code
        requests.put.return_value = return_mock

    def test_plot_registered_success(self):
        requests.post = MagicMock()
        self.assertTrue(SubMonitor().plot_registered)

    def test_plot_registered_failure(self):
        requests.post = MagicMock(
            side_effect=requests.exceptions.ConnectionError)
        self.assertFalse(SubMonitor().plot_registered)

    def test_flush_200(self):
        self._mock_requests_with_code(200)
        monitor = SubMonitor()
        monitor.last_samples = [[1, 2], [3, 4]]
        monitor.flush()
        self.assertEqual(requests.put.mock_calls[0][2]['json']['measures'],
                         [[1, 2], [3, 4]])
        self.assertEqual(requests.put.mock_calls[0][2]['json']['id'],
                         'SubMonitor_computerA')
        self.assertEqual(monitor.last_samples, [])

    def test_flush_404(self):
        self._mock_requests_with_code(404)
        monitor = SubMonitor()
        monitor.last_samples = [[1, 2], [3, 4]]
        monitor.flush()
        self.assertEqual(requests.put.mock_calls[0][2]['json']['measures'],
                         [[1, 2], [3, 4]])
        self.assertEqual(requests.put.mock_calls[0][2]['json']['id'],
                         'SubMonitor_computerA')
        self.assertEqual(monitor.last_samples, [[1, 2], [3, 4]])

if __name__ == '__main__':
    import sys
    sys.exit(unittest.main())



# -*- coding: utf-8 -*-
import sys
import socket

from remotestatsclient import RemoteStatsClient


def main():
    if not (2 <= len(sys.argv) <= 4):
        print('RemoteStats (client)')
        print('')
        print('usage: python {} <http://server_address> [ hostname '
              '[interval] ]'.format(sys.argv[0]))
        print('')
        print('hostname: defaults to the computer name')
        print('interval: defaults to 5s')
        sys.exit(1)

    server = sys.argv[1]
    hostname = sys.argv[2] if len(sys.argv) > 2 else socket.gethostname()
    interval = int(sys.argv[3]) if len(sys.argv) > 3 else 5
    RemoteStatsClient(server, hostname, interval).run()

if __name__ == '__main__':
    main()
